<?php
require_once('facebook_app.php');
require_once('config.php');

abstract class Editor {
    // + Factory implementation
    private static $instances = array();
    public static function factory($category, $id, $contenttype = null) {
        if(!isset(self::$instances[$category][$id])) {
            if(!isset($contenttype)) {
                $contenttype = Category::registry($category)->getContenttype();
            }
            $classname = 'Editor_' . $contenttype;
            self::$instances[$category][$id] = new $classname($category, $id, $contenttype);
        }
        return self::$instances[$category][$id];
    }
    private function __construct($category, $id, $contenttype) {
        $this->id = $id;
        $this->category = $category;
        $this->contenttype = $contenttype;
    }
    
    // - Factory implementation
    
    protected $id;
    protected $category;
    protected $contenttype;
    
    //
    //  Depends on the ui script loading editor.js and editor.css. Make sure to add them to display_header()!
    //
    public function getHtml() {
        $title = ($this->titleEnabled()) ? '<div class="editortitle"><input type="text" name="title" data-hint="Choose a title" /></div>' : '';
        $wall = ($this->wallEnabled()) ? '<label class="facebook">
                    <input type="checkbox" id="editor_chk_publish_facebook" name="chk_publish_facebook" value="" checked="checked" />
                    <span>Publish to Facebook Wall</span>
                </label>' : '';
        return 
        '<div id="editor_form">            
            <div class="loading"></div>
            <p class="editorheader">'.$this->header().'</p>
            '.$title.'
            <div class="textholder"><textarea class="text" data-hint="Enter a message"></textarea></div>
            <div class="buttons">
                <a onclick="Editor.cancel();" class="cancel"><span>Cancel</span></a>
                <a onclick="Editor.submit();" class="submit"><span>'.$this->submit().'</span></a>'
                .$wall.'
            </div>
            <div class="error"></div>
            <input type="hidden" name="id" value="'.$this->id.'" />
            <input type="hidden" name="contenttype" value="'.$this->contenttype.'" />
            <input type="hidden" name="category" value="'.$this->category.'" />
            <a name="editor"></a>
            <div class="clear"></div>
        </div>
    ';
    }
    
    protected function send_to_wall($attachment) {
        try {
            FacebookApp::instance()->getFacebookInterface()->api('/'.$_SESSION['userid'].'/feed', 'POST', $attachment);
        } catch(FacebookApiException $e) {
            error_log("facebook_error: " . $e);
        }
    }
    
    public function getContenttype() {
        return Category::registry($this->category)->getContenttype();
    }
    
    //
    //  editor_send.php consumes this for each particular content type, it expects 
    //  back json({ redirect : 'url' }) for general success, 0 for general failure.
    //
    //  For specific validation errors, it expects a json array with the first element 
    //  populated with another json array with the first element the error message and 
    //  the following elements the error values. (see vb api documents for details)    
    //
    public abstract function send($title, $message);
    
    
    // UI values for concrete types
    protected abstract function header();
    protected abstract function submit();
    public abstract function wall($title, $message);
    public abstract function wallEnabled();
    public abstract function titleEnabled();   
}
abstract class Base_Editor_Thread extends Editor {
    protected $thread_id;
    protected $redirect;
    public function send($title, $message) {
        $json = $this->doSend($title, $message);
        if(!empty($json['response']['postpreview']['errorlist']['errors'])) {
            return json_encode($json['response']['postpreview']['errorlist']['errors']);
        }
        if((!empty($json['response']['errormessage'])) && ($json['response']['errormessage'] == 'redirect_postthanks' || $json['response']['errormessage'] == 'redirect_postthanks_moderate')) {           
            $json2 = json_decode(VBApiClient::instance()->call_cached('showthread', array('threadid' => $json['show']['threadid'], 'apitextformat' => 1), array()), true);
            $this->thread_id = $json['show']['threadid'];
            $thread = ThreadInfo::get($this->thread_id);
            $thread->setSubscribed(@$json2['show']['subscribed'] == 1);
            $thread->setName($json2['response']['thread']['title']);
            ThreadInfo::set($thread);
            $this->redirect = 'category_item.php?category='. $this->category .'&id='.$this->thread_id.'&page='. (isset($json2['response']['pagenav']) ? $json2['response']['pagenav']['totalpages'] : 1).'&goto=newpost';
            return json_encode(array("redirect" => $this->redirect));
        } else {
            return json_encode($json);
        }
    }    
    public function wall($title, $message) {
        $opt = array();
        $opt['message'] = $this->wallMessage();
        $opt['link'] = FACEBOOK_CANVAS_URL . $this->redirect; 
        $opt['name'] = ThreadInfo::get($this->thread_id)->getName();
        $opt['caption'] = '';
        $opt['description'] = '';
        $this->send_to_wall($opt);
    }
    
    protected abstract function wallMessage();
    protected abstract function doSend($title, $message);
}
final class Editor_Threads extends Base_Editor_Thread {
    protected function header() {
        return 'Post a Reply';
    }
    protected function submit() {
        return 'Submit Reply';
    }
    public function wallEnabled() {
        return true;
    }
    public function titleEnabled() {
        return false;
    }
    protected function wallMessage() {
        return 'posted a reply to a thread on ' . FACEBOOK_APP_NAME;
    }
    protected function doSend($title, $message) {
        return json_decode(VBApiClient::instance()->call_cached('newreply_postreply', array(), array('threadid' => $this->id, 'message' => $message, 'subscribe' => (ThreadInfo::get($this->id)->isSubscribed() ? 1 : 0), 'loggedinuser' => VBApiClient::instance()->getUserid(), 'parseurl' => true)), true);
    }
}

final class Editor_Forums extends Base_Editor_Thread {
    protected function header() {
        return 'Create a thread';
    }
    protected function submit() {
        return 'Create Thread';
    }
    public function wallEnabled() {
        return true;
    }
    public function titleEnabled() {
        return true;
    }
    protected function wallMessage() {
        return 'started a new thread on ' . FACEBOOK_APP_NAME;
    }
    protected function doSend($title, $message) {
        return json_decode(VBApiClient::instance()->call_cached('newthread_postthread', array(), array('forumid' => $this->id, 'message' => $message, 'subject' => $title, 'loggedinuser' => VBApiClient::instance()->getUserid())), true);
    }
}
?>
